@@ -74,6 +74,9 @@ void EglOnXBackend::init()
     glPlatform->detect(EglPlatformInterface);
     if (GLPlatform::instance()->driver() == Driver_Intel)
         options->setUnredirectFullscreen(false); // bug #252817
+    options->setGlPreferBufferSwap(options->glPreferBufferSwap()); // resolve autosetting
+    if (options->glPreferBufferSwap() == Options::AutoSwapStrategy)
+        options->setGlPreferBufferSwap('e'); // for unknown drivers - should not happen
     glPlatform->printResults();
     initGL(EglPlatformInterface);
     if (!hasGLExtension("GL_OES_EGL_image")) {
@@ -174,7 +177,6 @@ bool EglOnXBackend::initRenderingContext()
     const EGLint context_attribs_31_core[] = {
         EGL_CONTEXT_MAJOR_VERSION_KHR, 3,
         EGL_CONTEXT_MINOR_VERSION_KHR, 1,
-        EGL_CONTEXT_FLAGS_KHR,         EGL_CONTEXT_OPENGL_FORWARD_COMPATIBLE_BIT_KHR,
         EGL_NONE
     };
 
@@ -277,6 +279,18 @@ void EglOnXBackend::present()
             eglWaitGL();
             if (char result = m_swapProfiler.end()) {
                 gs_tripleBufferUndetected = gs_tripleBufferNeedsDetection = false;
+                if (result == 'd' && GLPlatform::instance()->driver() == Driver_NVidia) {
+                    // TODO this is a workaround, we should get __GL_YIELD set before libGL checks it
+                    if (qstrcmp(qgetenv("__GL_YIELD"), "USLEEP")) {
+                        options->setGlPreferBufferSwap(0);
+                        eglSwapInterval(dpy, 0);
+                        kWarning(1212) << "\nIt seems you are using the nvidia driver without triple buffering\n"
+                                          "You must export __GL_YIELD=\"USLEEP\" to prevent large CPU overhead on synced swaps\n"
+                                          "Preferably, enable the TripleBuffer Option in the xorg.conf Device\n"
+                                          "For this reason, the tearing prevention has been disabled.\n"
+                                          "See https://bugs.kde.org/show_bug.cgi?id=322060\n";
+                    }
+                }
                 setBlocksForRetrace(result == 'd');
             }
         }
