@@ -97,6 +97,9 @@ void GlxBackend::init()
     glPlatform->detect(GlxPlatformInterface);
     if (GLPlatform::instance()->driver() == Driver_Intel)
         options->setUnredirectFullscreen(false); // bug #252817
+    options->setGlPreferBufferSwap(options->glPreferBufferSwap()); // resolve autosetting
+    if (options->glPreferBufferSwap() == Options::AutoSwapStrategy)
+        options->setGlPreferBufferSwap('e'); // for unknown drivers - should not happen
     glPlatform->printResults();
     initGL(GlxPlatformInterface);
     // Check whether certain features are supported
@@ -152,7 +155,7 @@ bool GlxBackend::initRenderingContext()
         const int attribs_31_core_robustness[] = {
             GLX_CONTEXT_MAJOR_VERSION_ARB,               3,
             GLX_CONTEXT_MINOR_VERSION_ARB,               1,
-            GLX_CONTEXT_FLAGS_ARB,                       GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB | GLX_CONTEXT_ROBUST_ACCESS_BIT_ARB,
+            GLX_CONTEXT_FLAGS_ARB,                       GLX_CONTEXT_ROBUST_ACCESS_BIT_ARB,
             GLX_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB, GLX_LOSE_CONTEXT_ON_RESET_ARB,
             0
         };
@@ -160,7 +163,6 @@ bool GlxBackend::initRenderingContext()
         const int attribs_31_core[] = {
             GLX_CONTEXT_MAJOR_VERSION_ARB, 3,
             GLX_CONTEXT_MINOR_VERSION_ARB, 1,
-            GLX_CONTEXT_FLAGS_ARB,         GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB,
             0
         };
 
@@ -432,6 +434,18 @@ void GlxBackend::present()
                 glXWaitGL();
                 if (char result = m_swapProfiler.end()) {
                     gs_tripleBufferUndetected = gs_tripleBufferNeedsDetection = false;
+                    if (result == 'd' && GLPlatform::instance()->driver() == Driver_NVidia) {
+                        // TODO this is a workaround, we should get __GL_YIELD set before libGL checks it
+                        if (qstrcmp(qgetenv("__GL_YIELD"), "USLEEP")) {
+                            options->setGlPreferBufferSwap(0);
+                            setSwapInterval(0);
+                            kWarning(1212) << "\nIt seems you are using the nvidia driver without triple buffering\n"
+                                              "You must export __GL_YIELD=\"USLEEP\" to prevent large CPU overhead on synced swaps\n"
+                                              "Preferably, enable the TripleBuffer Option in the xorg.conf Device\n"
+                                              "For this reason, the tearing prevention has been disabled.\n"
+                                              "See https://bugs.kde.org/show_bug.cgi?id=322060\n";
+                        }
+                    }
                     setBlocksForRetrace(result == 'd');
                 }
             }
